@@ -101,6 +101,7 @@
   (let* ((table (list nil))
 	 (tail table)
 	 (block-names '(("^<CJK Ideograph" . CJK\ IDEOGRAPH)
+                        ("^<Tangut Ideograph" . TANGUT\ IDEOGRAPH)
 			("^<Hangul Syllable" . HANGUL\ SYLLABLE)
 			("^<.*High Surrogate" . HIGH\ SURROGATE)
 			("^<.*Low Surrogate" . LOW\ SURROGATE)
@@ -779,6 +780,8 @@ Property value is a symbol `o' (Open), `c' (Close), or `n' (None)."
 			   (aref (aref jamo-name-table 2) (1- T)))))))
 	    ((eq sym 'CJK\ IDEOGRAPH)
 	     (format "%s-%04X" sym char))
+	    ((eq sym 'TANGUT\ IDEOGRAPH)
+	     (format "%s-%04X" sym char))
 	    ((eq sym 'CJK\ COMPATIBILITY\ IDEOGRAPH)
 	     (format "%s-%04X" sym char))
 	    ((eq sym 'HIGH\ SURROGATE)
@@ -1302,6 +1305,7 @@ Property value is a symbol `o' (Open), `c' (Close), or `n' (None)."
 	    unidata-text-file (or (pop command-line-args-left)
 				  (expand-file-name "unidata.txt"))))
   (let ((coding-system-for-write 'utf-8-unix)
+        (coding-system-for-read 'utf-8)
 	(charprop-file (expand-file-name "charprop.el" dest-dir))
 	(unidata-dir data-dir))
     (dolist (elt unidata-prop-alist)
