@@ -108,7 +108,7 @@ Point is at the end of the segment of this line within the rectangle."
 
 (defun rectangle--col-pos (col kind)
   (let ((c (move-to-column col)))
-    (if (and (= c col) (not (eolp)))
+    (if (= c col)
         (if (eq kind 'point)
             (if (window-parameter nil 'rectangle--point-crutches)
                 (setf (window-parameter nil 'rectangle--point-crutches) nil))
