@@ -59,7 +59,8 @@ region.")
 (make-obsolete-variable 'skeleton-autowrap nil "24.5")
 
 (defvar skeleton-end-newline t
-  "If non-nil, make sure that the skeleton inserted ends with a newline.")
+  "If non-nil, make sure that the skeleton inserted ends with a newline.
+This just influences the way the default `skeleton-end-hook' behaves.")
 
 (defvar skeleton-end-hook nil
   "Hook called at end of skeleton but before going to point of interest.
@@ -186,10 +187,6 @@ The optional third argument STR, if specified, is the value for the
 variable `str' within the skeleton.  When this is non-nil, the
 interactor gets ignored, and this should be a valid skeleton element.
 
-When done with skeleton, but before going back to `_'-point, add
-a newline (unless `skeleton-end-newline' is nil) and run the hook
-`skeleton-end-hook'.
-
 SKELETON is made up as (INTERACTOR ELEMENT ...).  INTERACTOR may be nil if
 not needed, a prompt-string or an expression for complex read functions.
 
@@ -238,7 +235,10 @@ available:
 		then: insert previously read string once more
 	help	help-form during interaction with the user or nil
 	input	initial input (string or cons with index) while reading str
-	v1, v2	local variables for memorizing anything you want"
+	v1, v2	local variables for memorizing anything you want
+
+When done with skeleton, but before going back to `_'-point call
+`skeleton-end-hook' if that is non-nil."
   (let ((skeleton-regions regions))
     (and skeleton-regions
 	 (setq skeleton-regions
