@@ -70,11 +70,7 @@ http://invisible-island.net/xterm/ctlseqs/ctlseqs.html)."
       (cond
        ((null event) nil)		;Unknown/bogus byte sequence!
        (is-down
-	(setf (terminal-parameter nil 'xterm-mouse-last-down)
-              ;; EVENT might be handed back to the input queue, which
-              ;; might modify it.  Copy it into the terminal parameter
-              ;; to guard against that.
-              (copy-sequence event))
+	(setf (terminal-parameter nil 'xterm-mouse-last-down) event)
 	vec)
        (is-move vec)
        (t
